﻿Imports System.Runtime.InteropServices
Imports System.ComponentModel

Public Class ReadStream
    Friend _outputs As New List(Of String)
    Private _AtEndOfLine As Boolean = False
    Private _Column As Integer = 0
    Private _Line As Integer = 0
    Friend _Shell As Shell

    Public Sub New(ByVal shell As Shell)
        Me._outputs = New List(Of String)
        Me._AtEndOfLine = False
        Me._Column = 0
        Me._Line = 0
        Me._Shell = shell
    End Sub

    ''' <summary>
    ''' AtEndOfLine
    '''     出力ストリーム内の行末に達したかどうかを示すブール値を返します。
    ''' </summary>
    Public ReadOnly Property AtEndOfLine() As Boolean
        Get
            Return _AtEndOfLine
        End Get
    End Property
    ''' <summary>
    ''' AtEndOfStream
    '''     出力ストリームの最後に達したかどうかを示すブール値を返します。
    ''' </summary>
    Public ReadOnly Property AtEndOfStream() As Boolean
        Get
            Try
                Return (_outputs.Count <= 0)
            Catch ex As System.Exception
                MsgBox("ReadStream.AtEndOfStream" & vbNewLine & ex.Message)
                Return True
            End Try
        End Get
    End Property
    ''' <summary>
    ''' Column
    '''     出力ストリームの現在の文字位置を示すカラム番号を返します。
    ''' </summary>
    Public ReadOnly Property Column() As Integer
        Get
            If _Column < 1 Then
                Return 1
            Else
                Return _Column
            End If
        End Get
    End Property
    ''' <summary>
    ''' Line
    '''     出力ストリームの現在の行番号を返します。
    ''' </summary>
    Public ReadOnly Property Line() As Integer
        Get
            Return _Line
        End Get
    End Property
    ''' <summary>
    ''' ReadAll
    '''     出力ストリーム全体を読み込み、結果の文字列を返します。
    ''' </summary>
    Public Function ReadAll() As String
        Dim strLines As String = ""
        SyncLock _Shell._EventLock
            strLines = String.Join(vbNewLine, _outputs)
            If _Column = 0 Then
                _Line += _outputs.Count
            Else
                _Line += _outputs.Count - 1
            End If
            _Column = 0
            _AtEndOfLine = True
            _outputs.Clear()
        End SyncLock
        Return strLines
    End Function
    ''' <summary>
    ''' ReadLine
    '''     出力ストリームから行全体を読み込み、結果の文字列を返します。
    ''' </summary>
    Public Function ReadLine() As String
        Dim strLine As String = ""
        SyncLock _Shell._EventLock
            If _outputs.Count > 0 Then
                strLine = _outputs.Item(0)
                _Line += 1
                _Column = 0
                _AtEndOfLine = True
                _outputs.RemoveAt(0)
            End If
        End SyncLock
        Return strLine
    End Function
    ''' <summary>
    ''' Read
    '''     指定された文字数を出力ストリームから読み込み、結果の文字列を返します。
    ''' </summary>
    ''' <param name="characters">文字数</param>
    Public Function Read(ByVal characters As Integer) As String
        Dim strText As String = ""
        Dim intCharCount As Integer = 0

        SyncLock _Shell._EventLock
            If _outputs.Count > 0 Then
                Do Until intCharCount >= characters Or _outputs.Count = 0
                    If Not String.IsNullOrEmpty(_outputs.Item(0)) Then
                        strText += _outputs.Item(0).Substring(0, 1)
                        intCharCount += 1
                        _outputs.Item(0) = _outputs.Item(0).Substring(1)
                        _Column += 1
                        If _Line = 0 Then _Line = 1
                        _AtEndOfLine = False
                    Else
                        _outputs.RemoveAt(0)
                        _Column = 0
                        _Line += 1
                        _AtEndOfLine = True
                    End If
                Loop
            Else
                _Column = 0
                _AtEndOfLine = True
            End If
        End SyncLock
        Return strText
    End Function

End Class
