﻿Imports System.Runtime.InteropServices
Imports System.ComponentModel
Imports System.Text


Public Class Shell

    Private _proc As System.Diagnostics.Process
    Private _StdIn As WriteStream
    Private _StdOut As ReadStream
    Private _StdErr As ReadStream
    Friend _EventLock As Object = New Object
    Friend _ticks As Long = 0
    Private _stopedSecond As Integer = 2    '終了判定秒数

    ''' <summary>
    ''' 非同期読込
    ''' </summary>
    Friend Class AsyncReadObject
        Friend _shell As Shell
        Friend _type As Byte
        Friend _esd As EventSeparaterDelegate
        Friend _rld As ReadLineDelegate
        Friend Property data As String
        Public Sub New(ByVal shell As Shell, ByVal type As Byte, ByVal esd As EventSeparaterDelegate, ByVal rld As ReadLineDelegate)
            Me._shell = shell   'メインクラス
            Me._type = type     'タイプ
            Me._esd = esd       'イベント振り分けデリケード
            Me._rld = rld       '出力読み取りデリケード
            Me.data = ""        '出力読込データ
        End Sub
    End Class

#Region "Public Command"
    ''' <summary>
    ''' Exec
    '''     子コマンドシェルでアプリケーションを実行します。
    ''' </summary>
    ''' <param name="command">実行コマンド</param>
    ''' <param name="windowStyle">ウインドウスタイル</param>
    ''' <param name="encoding">ウインドウスタイル</param>
    Public Function Exec(ByVal command As String, ByVal windowStyle As Short, Optional ByVal encoding As String = "") As Integer

        If Not _proc Is Nothing Then
            '----- プロセス終了まで待機
            _proc.WaitForExit()
            _proc.Close()
        End If

        _proc = New System.Diagnostics.Process()
        Dim psi As New System.Diagnostics.ProcessStartInfo()

        '----- コマンドからファイルパスと引数を設定
        Call ArgumentsSeparation(command, psi.FileName, psi.Arguments)

        '----- 実行時パラメータ設定
        'コマンドプロンプト画面の表示 
        Select Case windowStyle
            Case vbHide
                psi.WindowStyle = System.Diagnostics.ProcessWindowStyle.Hidden
                psi.CreateNoWindow = True
            Case vbMinimizedFocus
                psi.WindowStyle = System.Diagnostics.ProcessWindowStyle.Minimized
                psi.CreateNoWindow = False
            Case vbMaximizedFocus
                psi.WindowStyle = System.Diagnostics.ProcessWindowStyle.Maximized
                psi.CreateNoWindow = False
            Case Else
                psi.WindowStyle = System.Diagnostics.ProcessWindowStyle.Normal
                psi.CreateNoWindow = False
        End Select
        'コマンドプロンプト画面での文字エンコード 
        Select Case encoding.ToUpper
            Case "UTF8", "UTF-8"
                psi.StandardOutputEncoding = System.Text.Encoding.UTF8
                psi.StandardErrorEncoding = System.Text.Encoding.UTF8
            Case "UTF16", "UTF-16"
                psi.StandardOutputEncoding = System.Text.Encoding.Unicode
                psi.StandardErrorEncoding = System.Text.Encoding.Unicode
            Case "UTF32", "UTF-32"
                psi.StandardOutputEncoding = System.Text.Encoding.UTF32
                psi.StandardErrorEncoding = System.Text.Encoding.UTF32
            Case "ASCII"
                psi.StandardOutputEncoding = System.Text.Encoding.ASCII
                psi.StandardErrorEncoding = System.Text.Encoding.ASCII
        End Select
        psi.RedirectStandardInput = True
        psi.RedirectStandardOutput = True
        psi.RedirectStandardError = True
        psi.UseShellExecute = False
        _proc.StartInfo = psi
        _StdIn = New WriteStream(Me)    '入力処理用クラス
        _StdOut = New ReadStream(Me)    '出力処理用クラス(標準)
        _StdErr = New ReadStream(Me)    '出力処理用クラス(エラー)

        '----- 実行
        Call QueryPerformanceCounter(_ticks)    '高分解能パフォーマンスカウンタによる現在時刻の取得
        Call _proc.Start()                      'プロセスの実行

        '----- 標準入力ストリームの設定
        _StdIn._StreamWriter = _proc.StandardInput

        '----- 非同期での出力とエラーの読み取り設定
        Dim esd As EventSeparaterDelegate = New EventSeparaterDelegate(AddressOf EventSeparater)    'イベント振り分け(イベント発生用)
        Dim orld As ReadLineDelegate = New ReadLineDelegate(AddressOf ReadLine)                     '標準出力読み取り
        Dim erld As ReadLineDelegate = New ReadLineDelegate(AddressOf ReadLine)                     'エラー出力読み取り
        Dim oaro As AsyncReadObject = New AsyncReadObject(Me, 1, esd, orld)                         '非同期読取情報クラス(標準)
        Dim earo As AsyncReadObject = New AsyncReadObject(Me, 2, esd, erld)                         '非同期読取情報クラス(エラー)
        orld.BeginInvoke(oaro, New AsyncCallback(AddressOf ReadEnd), oaro)                          '標準出力読み取りを非同期で処理
        erld.BeginInvoke(earo, New AsyncCallback(AddressOf ReadEnd), earo)                          'エラー出力読み取りを非同期で処理

        Return _proc.Id

    End Function

    ''' <summary>
    ''' Terminate
    '''     開始したプロセスを終了するようにスクリプト・エンジンに指示する
    ''' </summary>
    Public Sub Terminate()
        If Not _proc Is Nothing Then
            Try
                _proc.Kill()            '実行したプロセスを強制終了する。
                _proc.WaitForExit()     '実行したプロセスが終了するまで待機する。
                _proc.Close()           'プロセスを開放する。
            Catch ex As System.Exception
            End Try
            _proc = Nothing
        End If
    End Sub
    ''' <summary>
    ''' Sleep
    '''     ミリ秒単位で一時停止。
    ''' </summary>
    Public Sub Sleep(ByVal millisecond As Integer)
        System.Threading.Thread.Sleep(millisecond)
    End Sub
    ''' <summary>
    ''' Second
    '''     秒取得
    ''' </summary>
    Private Function Second() As Double
        Static lpFrequency As Long = 0
        Dim ticks As Long = 0
        '----- カウンタ周波数取得(マシン上で測れる最大ミリ秒間隔)
        If lpFrequency = 0 Then Call QueryPerformanceFrequency(lpFrequency)
        '----- 高分解能パフォーマンスカウンタによる現在時刻の取得
        Call QueryPerformanceCounter(ticks)
        '----- 秒変換して返答
        Return (ticks / lpFrequency)
    End Function

#End Region

#Region "Private Command"
    ''' <summary>
    ''' ArgumentsSeparation
    '''     コマンドをファイルパスと引数に分割
    ''' </summary>
    ''' <param name="strCommand">実行コマンド</param>
    ''' <param name="strPath">ファイルパス</param>
    ''' <param name="strArguments">引数</param>
    Private Sub ArgumentsSeparation(ByVal strCommand As String, ByRef strPath As String, ByRef strArguments As String)
        Dim strSep As String = ""
        Dim intCharIndex As Integer
        strPath = strCommand
        strArguments = ""

        strCommand = LTrim(strCommand)
        strSep = Left(strCommand, 1)
        Select Case strSep
            Case "'", """"
            Case Else
                strSep = " "
        End Select
        For intCharIndex = 2 To strCommand.Length
            If Mid(strCommand, intCharIndex, 1) = strSep Then
                strPath = Left(strCommand, intCharIndex)
                strArguments = LTrim(Mid(strCommand, intCharIndex + 1))
                Exit For
            End If
        Next intCharIndex
    End Sub

    ''' <summary>
    ''' ReadLine
    '''     出力読み取り
    '''     標準・エラー出力のどちらかが行われた場合非同期で呼び出される
    '''     出力処理用クラスに出力内容を格納するのは
    '''     1. 先に実行プログラムが終了しても出力内容を保持する為
    '''     2. 出力内容を全行・1行・1文字単位で取得できるようにする為
    ''' </summary>
    ''' <param name="aro">AsyncReadObject</param>
    Friend Delegate Sub ReadLineDelegate(ByVal aro As AsyncReadObject)
    Friend Sub ReadLine(ByVal aro As AsyncReadObject)
        Select Case aro._type
            Case 1  '標準出力
                Call QueryPerformanceCounter(aro._shell._ticks)             '処理イベント時刻更新
                aro.data = aro._shell._proc.StandardOutput.ReadLine()       '標準出力を非同期読取情報クラス(標準)に登録(イベントで使用)
                _StdOut._outputs.Add(aro.data)                              '標準出力を出力処理用クラス(標準)に登録
            Case 2  'エラー出力
                Call QueryPerformanceCounter(aro._shell._ticks)             '処理イベント時刻更新
                aro.data = aro._shell._proc.StandardError.ReadLine()        'エラー出力を非同期読取情報クラス(エラー)に登録(イベントで使用)
                _StdErr._outputs.Add(aro.data)                              'エラー出力を出力処理用クラス(エラー)に登録
        End Select
    End Sub

    ''' <summary>
    ''' ReadEnd
    '''     出力読み取り完了
    ''' </summary>
    ''' <param name="iar">非同期ステータス</param>
    Friend Sub ReadEnd(ByVal iar As IAsyncResult)
        Dim aro As AsyncReadObject = CType(iar.AsyncState, AsyncReadObject)
        aro._rld.EndInvoke(iar)
        Select Case aro._type
            Case 1  '標準出力
                aro._esd.Invoke(1, aro.data)    'イベント振り分け実行(標準)
                Try
                    If Not aro._shell._proc.StandardOutput.EndOfStream Then
                        aro._rld.BeginInvoke(aro, New AsyncCallback(AddressOf ReadEnd), aro)    '標準出力読み取りを非同期で処理
                    End If
                Catch ex As System.Exception
                End Try

            Case 2  'エラー出力
                aro._esd.Invoke(2, aro.data)    'イベント振り分け実行(エラー)
                Try
                    If Not aro._shell._proc.StandardError.EndOfStream Then
                        aro._rld.BeginInvoke(aro, New AsyncCallback(AddressOf ReadEnd), aro)    'エラー出力読み取りを非同期で処理
                    End If
                Catch ex As System.Exception
                End Try
        End Select
    End Sub

    ''' <summary>
    ''' EventSeparater
    '''     イベント振り分け
    ''' </summary>
    ''' <param name="type">イベントタイプ</param>
    ''' <param name="data">データ</param>
    Friend Delegate Sub EventSeparaterDelegate(ByVal type As Byte, ByVal data As Object)
    Friend Sub EventSeparater(ByVal type As Byte, ByVal data As Object)
        Try
            Select Case type
                Case 1  '標準出力
                    '----- 標準出力イベントの発生
                    RaiseEvent OutputDataReceived(data)
                Case 2  'エラー出力
                    '----- エラー出力イベントの発生
                    RaiseEvent ErrorDataReceived(data)
            End Select
        Catch ex As System.Exception
        End Try
    End Sub

#End Region

#Region "Public Property"
    ''' <summary>
    ''' ExitCode
    '''     実行したスクリプトまたはプログラムが設定した終了コードを返す
    ''' </summary>
    Friend ReadOnly Property ExitCode() As Short
        Get
            If Not _proc Is Nothing Then
                Return _proc.ExitCode
            Else
                Return 0
            End If
        End Get
    End Property
    ''' <summary>
    ''' ProcessID
    '''     WIShScriptExecオブジェクトによって起動されたプロセスのプロセスID（PID）
    ''' </summary>
    Friend ReadOnly Property ProcessID() As Integer
        Get
            If Not _proc Is Nothing Then
                Return _proc.Id
            Else
                Return 0
            End If
        End Get
    End Property
    ''' <summary>
    ''' Status
    '''     実行したスクリプトのステータス情報を返す
    ''' </summary>
    Friend ReadOnly Property Status() As Short
        Get
            If Not _proc Is Nothing Then
                If _proc.HasExited Then
                    Return 1
                Else
                    Return 0
                End If
            Else
                Return 0
            End If
        End Get
    End Property
    ''' <summary>
    ''' StdIn
    '''     標準入力ストリーム
    ''' </summary>
    Friend ReadOnly Property StdIn() As WriteStream
        Get
            Return _StdIn
        End Get
    End Property
    ''' <summary>
    ''' StdOut
    '''     標準出力ストリーム
    ''' </summary>
    Friend ReadOnly Property StdOut() As ReadStream
        Get
            Return _StdOut
        End Get
    End Property
    ''' <summary>
    ''' StdErr
    '''     エラー出力ストリーム
    ''' </summary>
    Friend ReadOnly Property StdErr() As ReadStream
        Get
            Return _StdErr
        End Get
    End Property
    ''' <summary>
    ''' HasStoped
    '''     処理の停止判定
    ''' </summary>
    Friend ReadOnly Property HasStoped() As Boolean
        Get
            Dim blnReturn As Boolean = False
            Dim ticks As Long = 0
            Static frequency As Long = 0
            If Not _proc Is Nothing Then
                If _proc.HasExited Then
                    '----- 実行終了している場合
                    blnReturn = True
                Else
                    '----- _stopedSecondで指定された秒以上出力がない場合
                    If frequency = 0 Then Call QueryPerformanceFrequency(frequency)
                    Call QueryPerformanceCounter(ticks)
                    '----- 一定時間経過判定
                    If ((ticks - _ticks) / frequency) >= _stopedSecond Then
                        blnReturn = True
                    End If
                End If
            Else
                blnReturn = True
            End If
            Return blnReturn
        End Get
    End Property
    ''' <summary>
    ''' StopedSecond
    '''     停止判定秒数
    ''' </summary>
    Friend Property StopedSecond() As Integer
        Get
            Return _stopedSecond
        End Get
        Set(ByVal value As Integer)
            _stopedSecond = value
        End Set
    End Property
    ''' <summary>
    ''' HasExited
    '''     処理の終了判定
    ''' </summary>
    Friend ReadOnly Property HasExited() As Boolean
        Get
            Dim blnReturn As Boolean = False
            If Not _proc Is Nothing Then
                If _proc.HasExited _
                AndAlso _proc.StandardOutput.EndOfStream _
                AndAlso _proc.StandardError.EndOfStream Then
                    '----- プロセスの解放
                    _proc.Close()
                    _proc = Nothing
                    blnReturn = True
                End If
            Else
                blnReturn = True
            End If
            Return blnReturn
        End Get
    End Property
#End Region

#Region "Public Event"
    ''' <summary>
    ''' OutputDataReceived
    '''     標準出力にデータが記載された時
    ''' </summary>
    ''' <param name="data">データ</param>
    Friend Delegate Sub outputDataReceivedDelegate(ByVal data As Object)
    Friend Event OutputDataReceived As outputDataReceivedDelegate

    ''' <summary>
    ''' ErrorDataReceived
    '''     エラー出力にデータが記載された時
    ''' </summary>
    ''' <param name="data">データ</param>
    Friend Delegate Sub errorDataReceivedDelegate(ByVal data As Object)
    Friend Event ErrorDataReceived As errorDataReceivedDelegate
#End Region
End Class
